﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows;
using Telerik.Windows.Controls.Diagrams;
using Telerik.Windows.Diagrams.Core;

namespace SilverlightApplication21
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			InitializeComponent();

			this.diagram.ConnectionManipulationCompleted += OnDiagramConnectionManipulationCompleted;
			this.diagram.ConnectionManipulationStarted += OnConnectionManipulationStarted;

			EventManager.RegisterClassHandler(typeof(RadDiagramConnector), RadDiagramConnector.ActivationChangedEvent, new RadRoutedEventHandler(OnConnectorActivationChanged));
		}

		private void OnConnectionManipulationStarted(object sender, ManipulationRoutedEventArgs e)
		{
			e.Handled = e.ManipulationStatus == ManipulationStatus.Attaching && !e.Connector.CanAttachNewConnection(1);
		}

		private void OnDiagramConnectionManipulationCompleted(object sender, ManipulationRoutedEventArgs e)
		{
			e.Handled = e.ManipulationStatus == ManipulationStatus.Attaching && !e.Connector.CanAttachNewConnection(1);
		}

		private static void OnConnectorActivationChanged(object sender, RadRoutedEventArgs e)
		{
			var connector = sender as IConnector;
			if (connector != null)
			{
				RadDiagramConnectorExtensions.SetIsConnectorValid((DependencyObject)connector, connector.IsActive && connector.CanAttachNewConnection(1) || !connector.IsActive);
			}
		}
	}

	public class RadDiagramConnectorExtensions : DependencyObject
	{
		public static readonly DependencyProperty IsConnectorValidProperty =
			DependencyProperty.RegisterAttached("IsConnectorValid", typeof(bool), typeof(RadDiagramConnectorExtensions), new Telerik.Windows.PropertyMetadata(true));

		public static bool GetIsConnectorValid(DependencyObject obj)
		{
			return (bool)obj.GetValue(IsConnectorValidProperty);
		}

		public static void SetIsConnectorValid(DependencyObject obj, bool value)
		{
			obj.SetValue(IsConnectorValidProperty, value);
		}
	}

	public static class RadDiagramConnectorHelpers
	{
		public static IEnumerable<IConnection> GetOutgoingLinks(this IConnector connector)
		{
			return connector.Shape.OutgoingLinks.Where(x => x.SourceConnectorResult == connector);
		}

		public static IEnumerable<IConnection> GetIncomingLinks(this IConnector connector)
		{
			return connector.Shape.IncomingLinks.Where(x => x.TargetConnectorResult == connector);
		}

		public static bool CanAttachNewConnection(this IConnector connector, int maxConnectionsCount)
		{
			return connector.GetOutgoingLinks().Count() + connector.GetIncomingLinks().Count() < maxConnectionsCount;
		}
	}
}
